# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/wordslab-notebooks-lib',
                'doc_host': 'https://wordslab-org.github.io',
                'git_url': 'https://github.com/wordslab-org/wordslab-notebooks-lib',
                'lib_path': 'wordslab_notebooks_lib'},
  'syms': { 'wordslab_notebooks_lib.chat': { 'wordslab_notebooks_lib.chat.ChatTurn': ( 'chat.html#chatturn',
                                                                                       'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ChatTurn.__init__': ( 'chat.html#chatturn.__init__',
                                                                                                'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ChatTurn.append_content': ( 'chat.html#chatturn.append_content',
                                                                                                      'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ChatTurn.append_thinking': ( 'chat.html#chatturn.append_thinking',
                                                                                                       'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ChatTurn.append_tool_call': ( 'chat.html#chatturn.append_tool_call',
                                                                                                        'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ChatTurn.content': ( 'chat.html#chatturn.content',
                                                                                               'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ChatTurn.end_tool_call': ( 'chat.html#chatturn.end_tool_call',
                                                                                                     'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ChatTurn.start_tool_call': ( 'chat.html#chatturn.start_tool_call',
                                                                                                       'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ChatTurn.thinking': ( 'chat.html#chatturn.thinking',
                                                                                                'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ChatTurn.to_markdown': ( 'chat.html#chatturn.to_markdown',
                                                                                                   'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ChatTurns': ( 'chat.html#chatturns',
                                                                                        'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ChatTurns.__init__': ( 'chat.html#chatturns.__init__',
                                                                                                 'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ChatTurns.new_turn': ( 'chat.html#chatturns.new_turn',
                                                                                                 'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.Images': ('chat.html#images', 'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.Images.__init__': ( 'chat.html#images.__init__',
                                                                                              'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.Images.add_bytes': ( 'chat.html#images.add_bytes',
                                                                                               'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.Images.add_file_path': ( 'chat.html#images.add_file_path',
                                                                                                   'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.Images.add_web_url': ( 'chat.html#images.add_web_url',
                                                                                                 'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.Images.get_base64_data': ( 'chat.html#images.get_base64_data',
                                                                                                     'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.Images.get_base64_data_or_url': ( 'chat.html#images.get_base64_data_or_url',
                                                                                                            'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.Images.is_web_url': ( 'chat.html#images.is_web_url',
                                                                                                'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ModelClient': ( 'chat.html#modelclient',
                                                                                          'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ModelClient.__call__': ( 'chat.html#modelclient.__call__',
                                                                                                   'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ModelClient.__init__': ( 'chat.html#modelclient.__init__',
                                                                                                   'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ModelClient._check_call_parameters_type': ( 'chat.html#modelclient._check_call_parameters_type',
                                                                                                                      'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ModelClient._manage_messages': ( 'chat.html#modelclient._manage_messages',
                                                                                                           'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.OllamaModelClient': ( 'chat.html#ollamamodelclient',
                                                                                                'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.OllamaModelClient.__call__': ( 'chat.html#ollamamodelclient.__call__',
                                                                                                         'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.OllamaModelClient.__init__': ( 'chat.html#ollamamodelclient.__init__',
                                                                                                         'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.OpenRouterModelClient': ( 'chat.html#openroutermodelclient',
                                                                                                    'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.OpenRouterModelClient.__call__': ( 'chat.html#openroutermodelclient.__call__',
                                                                                                             'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.OpenRouterModelClient.__init__': ( 'chat.html#openroutermodelclient.__init__',
                                                                                                             'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.ToolExecutionError': ( 'chat.html#toolexecutionerror',
                                                                                                 'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.Tools': ('chat.html#tools', 'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.Tools.__init__': ( 'chat.html#tools.__init__',
                                                                                             'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.Tools.call': ( 'chat.html#tools.call',
                                                                                         'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.Tools.get_function': ( 'chat.html#tools.get_function',
                                                                                                 'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.Tools.get_functions': ( 'chat.html#tools.get_functions',
                                                                                                  'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.Tools.get_schema': ( 'chat.html#tools.get_schema',
                                                                                               'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.Tools.get_schemas': ( 'chat.html#tools.get_schemas',
                                                                                                'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.Tools.has_tool': ( 'chat.html#tools.has_tool',
                                                                                             'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat._get_function_schema': ( 'chat.html#_get_function_schema',
                                                                                                   'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat._messages_words': ( 'chat.html#_messages_words',
                                                                                              'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.get_tools_schemas_and_functions': ( 'chat.html#get_tools_schemas_and_functions',
                                                                                                              'wordslab_notebooks_lib/chat.py'),
                                             'wordslab_notebooks_lib.chat.refresh_notebook_display': ( 'chat.html#refresh_notebook_display',
                                                                                                       'wordslab_notebooks_lib/chat.py')},
            'wordslab_notebooks_lib.core': { 'wordslab_notebooks_lib.core.Wordslab': ( 'core.html#wordslab',
                                                                                       'wordslab_notebooks_lib/core.py'),
                                             'wordslab_notebooks_lib.core.Wordslab.__init__': ( 'core.html#wordslab.__init__',
                                                                                                'wordslab_notebooks_lib/core.py')},
            'wordslab_notebooks_lib.env': { 'wordslab_notebooks_lib.env.WordslabEnv': ( 'env.html#wordslabenv',
                                                                                        'wordslab_notebooks_lib/env.py'),
                                            'wordslab_notebooks_lib.env.WordslabEnv.__init__': ( 'env.html#wordslabenv.__init__',
                                                                                                 'wordslab_notebooks_lib/env.py'),
                                            'wordslab_notebooks_lib.env.WordslabEnv.read_user_env_var': ( 'env.html#wordslabenv.read_user_env_var',
                                                                                                          'wordslab_notebooks_lib/env.py'),
                                            'wordslab_notebooks_lib.env.WordslabEnv.setup_github': ( 'env.html#wordslabenv.setup_github',
                                                                                                     'wordslab_notebooks_lib/env.py'),
                                            'wordslab_notebooks_lib.env.WordslabEnv.setup_huggingface': ( 'env.html#wordslabenv.setup_huggingface',
                                                                                                          'wordslab_notebooks_lib/env.py'),
                                            'wordslab_notebooks_lib.env.WordslabEnv.setup_ollamacloud': ( 'env.html#wordslabenv.setup_ollamacloud',
                                                                                                          'wordslab_notebooks_lib/env.py'),
                                            'wordslab_notebooks_lib.env.WordslabEnv.setup_openrouter': ( 'env.html#wordslabenv.setup_openrouter',
                                                                                                         'wordslab_notebooks_lib/env.py'),
                                            'wordslab_notebooks_lib.env.WordslabEnv.setup_pypi': ( 'env.html#wordslabenv.setup_pypi',
                                                                                                   'wordslab_notebooks_lib/env.py'),
                                            'wordslab_notebooks_lib.env.WordslabEnv.setup_replicate': ( 'env.html#wordslabenv.setup_replicate',
                                                                                                        'wordslab_notebooks_lib/env.py'),
                                            'wordslab_notebooks_lib.env.WordslabEnv.setup_tavily': ( 'env.html#wordslabenv.setup_tavily',
                                                                                                     'wordslab_notebooks_lib/env.py'),
                                            'wordslab_notebooks_lib.env.WordslabEnv.write_user_env_var': ( 'env.html#wordslabenv.write_user_env_var',
                                                                                                           'wordslab_notebooks_lib/env.py'),
                                            'wordslab_notebooks_lib.env._expand_with_current_env': ( 'env.html#_expand_with_current_env',
                                                                                                     'wordslab_notebooks_lib/env.py')},
            'wordslab_notebooks_lib.notebook': { 'wordslab_notebooks_lib.notebook.Inspector._get_info': ( 'notebook.html#inspector._get_info',
                                                                                                          'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.InteractiveShell.get_tools_schemas_and_functions': ( 'notebook.html#interactiveshell.get_tools_schemas_and_functions',
                                                                                                                                       'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.InteractiveShell.get_variables_values': ( 'notebook.html#interactiveshell.get_variables_values',
                                                                                                                            'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.InteractiveShell.user_items': ( 'notebook.html#interactiveshell.user_items',
                                                                                                                  'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook': ( 'notebook.html#wordslabnotebook',
                                                                                                       'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.JupyterlabExtensionComm': ( 'notebook.html#wordslabnotebook.jupyterlabextensioncomm',
                                                                                                                               'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.__init__': ( 'notebook.html#wordslabnotebook.__init__',
                                                                                                                'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook._ensure_jupyterlab_extension': ( 'notebook.html#wordslabnotebook._ensure_jupyterlab_extension',
                                                                                                                                    'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.add_cell': ( 'notebook.html#wordslabnotebook.add_cell',
                                                                                                                'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.cell_id': ( 'notebook.html#wordslabnotebook.cell_id',
                                                                                                               'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.chat': ( 'notebook.html#wordslabnotebook.chat',
                                                                                                            'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.content': ( 'notebook.html#wordslabnotebook.content',
                                                                                                               'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.delete_cell': ( 'notebook.html#wordslabnotebook.delete_cell',
                                                                                                                   'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.get_context_for_llm': ( 'notebook.html#wordslabnotebook.get_context_for_llm',
                                                                                                                           'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.get_tools_schemas_and_functions': ( 'notebook.html#wordslabnotebook.get_tools_schemas_and_functions',
                                                                                                                                       'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.get_variables_values': ( 'notebook.html#wordslabnotebook.get_variables_values',
                                                                                                                            'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.jupyterlab_extension_version': ( 'notebook.html#wordslabnotebook.jupyterlab_extension_version',
                                                                                                                                    'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.path': ( 'notebook.html#wordslabnotebook.path',
                                                                                                            'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.read_cell': ( 'notebook.html#wordslabnotebook.read_cell',
                                                                                                                 'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.run_cell': ( 'notebook.html#wordslabnotebook.run_cell',
                                                                                                                'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.set_ollama_chat_model': ( 'notebook.html#wordslabnotebook.set_ollama_chat_model',
                                                                                                                             'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.set_openrouter_chat_model': ( 'notebook.html#wordslabnotebook.set_openrouter_chat_model',
                                                                                                                                 'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.show_object_members': ( 'notebook.html#wordslabnotebook.show_object_members',
                                                                                                                           'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.show_variables_and_functions': ( 'notebook.html#wordslabnotebook.show_variables_and_functions',
                                                                                                                                    'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.WordslabNotebook.update_cell': ( 'notebook.html#wordslabnotebook.update_cell',
                                                                                                                   'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook._cell_output_to_xml': ( 'notebook.html#_cell_output_to_xml',
                                                                                                          'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook._cell_to_xml': ( 'notebook.html#_cell_to_xml',
                                                                                                   'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook._cells_to_notebook_xml': ( 'notebook.html#_cells_to_notebook_xml',
                                                                                                             'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook._find_frame_dict': ( 'notebook.html#_find_frame_dict',
                                                                                                       'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook._get_schema': ( 'notebook.html#_get_schema',
                                                                                                  'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook._mime_bundle_to_text': ( 'notebook.html#_mime_bundle_to_text',
                                                                                                           'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook._safe_attr_doc': ( 'notebook.html#_safe_attr_doc',
                                                                                                     'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook._safe_getattr': ( 'notebook.html#_safe_getattr',
                                                                                                    'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook._safe_str': ( 'notebook.html#_safe_str',
                                                                                                'wordslab_notebooks_lib/notebook.py'),
                                                 'wordslab_notebooks_lib.notebook.find_var': ( 'notebook.html#find_var',
                                                                                               'wordslab_notebooks_lib/notebook.py')},
            'wordslab_notebooks_lib.tools': { 'wordslab_notebooks_lib.tools._absolutify_imgs': ( 'tools.html#_absolutify_imgs',
                                                                                                 'wordslab_notebooks_lib/tools.py'),
                                              'wordslab_notebooks_lib.tools._convert_math': ( 'tools.html#_convert_math',
                                                                                              'wordslab_notebooks_lib/tools.py'),
                                              'wordslab_notebooks_lib.tools.create': ( 'tools.html#create',
                                                                                       'wordslab_notebooks_lib/tools.py'),
                                              'wordslab_notebooks_lib.tools.insert': ( 'tools.html#insert',
                                                                                       'wordslab_notebooks_lib/tools.py'),
                                              'wordslab_notebooks_lib.tools.read_url': ( 'tools.html#read_url',
                                                                                         'wordslab_notebooks_lib/tools.py'),
                                              'wordslab_notebooks_lib.tools.scrape_url': ( 'tools.html#scrape_url',
                                                                                           'wordslab_notebooks_lib/tools.py'),
                                              'wordslab_notebooks_lib.tools.str_replace': ( 'tools.html#str_replace',
                                                                                            'wordslab_notebooks_lib/tools.py'),
                                              'wordslab_notebooks_lib.tools.view': ('tools.html#view', 'wordslab_notebooks_lib/tools.py')}}}
